﻿using System.Drawing;


namespace WindowsFormsApplication1
{
	internal class Fractal
	{
		public int[,] iterations;
        public System.Windows.Point centerPoint = new System.Windows.Point(-1.0, 0);
        public int xSize = 100, ySize = 100;
		public int maxIterations = 100;
       
        public double maxX = 0.8;
        public double minX = -2.2;
        public double maxY = 1.1;
        public double minY = -1.1;


        public Color SetColorRGB(int x, int y)
        {
            double colorPart1 = maxIterations / 15.0;
            double colorPart2 = 2.0 * colorPart1;
            double colorPart3 = 4.0 * colorPart1;
            double colorPart4 = 8.0 * colorPart1;
            Color pix;
            int B;
            int G;
            int R;

            if (iterations[x, y] >= maxIterations)
            {
                pix = Color.Black;
            }
            else
            {
                if (iterations[x, y] < colorPart1)
                {
                    B = (int)(0xff * (iterations[x, y] / colorPart1));
                    G = 0;
                    R = 0;
                }
                else
                {
                    if (iterations[x, y] < colorPart2)
                    {
                        B = 0xFF - (int)(0xff * ((iterations[x, y] - colorPart1) / colorPart1));
                        G = (int)(0xff * ((iterations[x, y] - colorPart1) / colorPart1));
                        R = 0;
                    }
                    else
                    {
                        if (iterations[x, y] < colorPart3)
                        {
                            B = 0;
                            G = 0xFF - (int)(0xff * ((iterations[x, y] - colorPart2) / colorPart2));
                            R = (int)(0xff * ((iterations[x, y] - colorPart2) / colorPart2));
                        }
                        else
                        {
                            if (iterations[x, y] < colorPart4)
                            {
                                B = 0;
                                G = (int)(0xff * ((iterations[x, y] - colorPart3) / colorPart3));
                                R = 0xff;
                            }
                            else
                            {
                                B = (int)(0xff * ((iterations[x, y] - colorPart4) / colorPart4));
                                G = 0xff;
                                R = 0xff;
                            }
                        }
                    }
                }
                if (R < 0)
                    R = 0;
                else if (R > 255)
                    R = 255;
                if (G < 0)
                    G = 0;
                else if (G > 255)
                    G = 255;
                if (B < 0)
                    B = 0;
                else if (B > 255)
                    B = 255;
                pix = Color.FromArgb(R, G, B);
            }
            return pix;
        }


        public Fractal(int sizeX, int sizeY)
		{
			xSize = sizeX;
			ySize = sizeY;
			iterations = new int[sizeX, sizeY];
		}

		
        public System.Windows.Point ScreenToMB(int xScreen, int yScreen)
        {
            System.Windows.Point ReturnVal = new System.Windows.Point();
            ReturnVal.X = minX + (maxX - minX) * xScreen / xSize;
            ReturnVal.Y = minY + (maxY - minY) * yScreen / ySize;
            return ReturnVal;
        }

        public void CreatePic()
		{
            System.Windows.Point Z = new System.Windows.Point();

			for (int x = 0; x < xSize; x++)
			{
                for (int y = 0; y < ySize; y++)
                {
                    Z= ScreenToMB(x, y);
                    int i = IterationsLoop(Z);
                    iterations[x, y] = i;
                }
			}
		}


        public int IterationsLoop(System.Windows.Point p)
        {
            System.Windows.Point z = new System.Windows.Point(0, 0);
            int i;
            double xtemp;
            for (i = 0; i <= maxIterations; i++)
            {
                xtemp = z.X * z.X - z.Y * z.Y + p.X;
                z.Y = 2 * z.X * z.Y + p.Y;
                z.X = xtemp;
                if (z.X * z.X + z.Y * z.Y > 4)
                    break;
            }
            return i;
        }
    }
}
