﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;

/*                   Plot Mandelbrot set                     */
/*                    www.mosismath.com                      */

namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        double tempX = 0, tempY = 0;
        double oldMaxX = 0, oldMaxY = 0;
        double oldMinX = 0, oldMinY = 0;
        int winULeftX, winULeftY, winBoRightX, winBoRightY;
        int oldWinBoRightX = 0, oldWinBoRightY = 0;
        bool drawWindow = false;
        bool zoomed = false;
        private Fractal fractal;
        private Bitmap tempBitmap;
        private Bitmap bmpDest;
        private Pen whitekPen = new Pen(Color.White, 3);
        private BackgroundWorker calculationThread;


        public MainWin()
        {     
            InitializeComponent();
            bmpDest = new Bitmap((Bitmap)pictureBox.Image);
            tempBitmap = (Bitmap)pictureBox.Image.Clone();
            pictureBox.Width = pictureBox.Image.Width;
            pictureBox.Height = pictureBox.Image.Height;
            fractal = new Fractal(pictureBox.Width, pictureBox.Height);
            oldMaxX = fractal.maxX;
            oldMaxY = fractal.maxY;
            oldMinX = fractal.minX;
            oldMinY = fractal.minY;
            tbIterations.Text = fractal.maxIterations.ToString();
            calculationThread = new BackgroundWorker();
            calculationThread.DoWork += calculate;
            calculationThread.RunWorkerCompleted += calculationCompleted;
        }


        private void calculate(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            winBoRightX = 0;
            winULeftX = 0;
            fractal.CreatePic();
            for (int y = 0; y < fractal.ySize; y++)
            {
                for (int x = 0; x < fractal.xSize; x++)
                {
                    bmpDest.SetPixel(x, y, fractal.SetColorRGB(x, y));
                }
            }
        }


        private void calculationCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            pictureBox.Image = (Bitmap)bmpDest.Clone();
            tempBitmap = (Bitmap)pictureBox.Image.Clone();
            zoomed = false;
            pictureBox.Image = (Bitmap)bmpDest.Clone();
            this.Cursor = Cursors.Default;
            Cursor.Current = Cursors.Default;
            btCalc.Enabled = true;
            btSave.Enabled = true;
            btReset.Enabled = true;
            pictureBox.Enabled = true;
            tbIterations.Enabled = true;
        }
      
	
        private void btReset_Click(object sender, EventArgs e)
        {
            fractal.maxX = 0.8;
            fractal.minX = -2.2;
            fractal.maxY = 1.1;
            fractal.minY = -1.1;
            winBoRightX = 0;
            winULeftX = 0;
            zoomed = false;
            pictureBox.Image = (Bitmap)tempBitmap.Clone();
        }


        private void btSaveClick(object sender, EventArgs e)
        {
            pictureBox.Image.Save(Application.StartupPath + "\\Fractal.Png", ImageFormat.Png);
        }


        private void btCalcClick(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            Cursor.Current = Cursors.WaitCursor;
            btCalc.Enabled = false;
            btSave.Enabled = false;
            btReset.Enabled = false;
            pictureBox.Enabled = false;
            tbIterations.Enabled = false;
            try
            {
                fractal.maxIterations = Convert.ToInt32(tbIterations.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show("MaxIterations \n\n" + ex.Message);
                fractal.maxIterations = 100;
            }
            calculationThread.RunWorkerAsync();
        }

       
        private void pictureBox2_Paint(object sender, PaintEventArgs e)
        {
            int x, y;
            try
            {
                if (winBoRightX - winULeftX > 5)
                {
                    // Clear old rectangle
                    for (x= winULeftX-1; x < oldWinBoRightX; x++)
                    {
                        for(y= winULeftY-1; y < oldWinBoRightY; y++)
                        {
                            ((Bitmap)pictureBox.Image).SetPixel(x, y, tempBitmap.GetPixel(x, y));
                        }
                    }
                    // Draw new zoom rectangle
                    using (var graphics = Graphics.FromImage((pictureBox.Image)))
                    {
                        graphics.DrawRectangle(whitekPen, winULeftX, winULeftY, winBoRightX - winULeftX, winBoRightY - winULeftY);
                    }
                    oldWinBoRightX = winBoRightX+2;
                    oldWinBoRightY = winBoRightY+2;
                    if (oldWinBoRightX > pictureBox.Image.Width)
                        oldWinBoRightX = pictureBox.Image.Width;
                    if (oldWinBoRightY > pictureBox.Image.Height)
                        oldWinBoRightY = pictureBox.Image.Height;
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
     

        private void pictureBox2_MouseMove(object sender, MouseEventArgs e)
        {
            try
            {
                if (drawWindow)
                {
                    winBoRightX = e.X;
                    winBoRightY = e.Y;
                    if (winBoRightX - winULeftX > 5)
                    {
                        pictureBox.Refresh();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }


        private void pictureBox2_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                if (drawWindow)
                {
                    // Save zoom rectangle
                    System.Windows.Point NewVal = fractal.ScreenToMB(e.X, e.Y);
                    zoomed = true;
                    if (NewVal.X > tempX)
                    {
                        fractal.maxX = NewVal.X;
                        fractal.minX = tempX;
                    }
                    else
                    {
                        fractal.maxX = tempX;
                        fractal.minX = NewVal.X;
                    }
                    if (NewVal.Y > tempY)
                    {
                        fractal.maxY = NewVal.Y;
                        fractal.minY = tempY;
                    }
                    else
                    {
                        fractal.maxY = tempY;
                        fractal.minY = NewVal.Y;
                    }
                    fractal.maxY = 3 * (fractal.maxX - fractal.minX) / 4 + fractal.minY;
                }
                drawWindow = false;
            }
        }

        private void pictureBox2_MouseDown(object sender, MouseEventArgs e)
        {

            winULeftX = e.X;
            winULeftY = e.Y;
            System.Windows.Point NewVal = fractal.ScreenToMB(e.X, e.Y);
            if (e.Button == MouseButtons.Left)
            {
                // Start zoom rectangle left top
                if (!zoomed)
                {
                    drawWindow = true;
                    tempX = NewVal.X;
                    tempY = NewVal.Y;
                    oldMaxX = fractal.maxX;
                    oldMaxY = fractal.maxY;
                    oldMinX = fractal.minX;
                    oldMinY = fractal.minY;
                }
            }
            else
            {
                // Reset zoom coordinates
                if (e.Button == MouseButtons.Right)
                {
                    fractal.maxX = oldMaxX;
                    fractal.maxY = oldMaxY;
                    fractal.minX = oldMinX;
                    fractal.minY = oldMinY;
                    winBoRightX = 0;
                    winULeftX = 0;
                    zoomed = false;
                    pictureBox.Image = (Bitmap)tempBitmap.Clone();
                }
            }
        }
        
    }
}
